package file

import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS

@TestInstance(PER_CLASS)
class FilePositionTrackerTest {

    companion object {
        const val FILE_NAME = "testFile"
    }

    private lateinit var trackerUnderTest: FilePositionTracker

    private fun createFilePosition(lineNumber: Int, columnNumber: Int): FilePosition {
        return FilePosition(FILE_NAME, lineNumber, columnNumber)
    }

    private fun assertTrackerPositionEquals(expectedPosition: FilePosition) {
        assertThat(trackerUnderTest.currentPosition()).isEqualTo(expectedPosition)
    }

    @BeforeEach
    fun setUp() {
        trackerUnderTest = FilePositionTracker(FILE_NAME)
    }

    @Test
    fun noMovement_ReturnsInitialPosition() {
        assertTrackerPositionEquals(createFilePosition(1, 0))
    }

    @Test
    fun advanceOneColumn_ReturnsCorrectPosition() {
        trackerUnderTest.incrementColumn()

        assertTrackerPositionEquals(createFilePosition(1, 1))
    }

    @Test
    fun advanceOneLine_ReturnsCorrectPosition() {
        trackerUnderTest.incrementLine()

        assertTrackerPositionEquals(createFilePosition(2, 0))
    }

    @Test
    fun advanceOneLine_ResetsColumn() {
        trackerUnderTest.incrementColumn()
        trackerUnderTest.incrementLine()

        assertTrackerPositionEquals(createFilePosition(2, 0))
    }

}
